@@ -1,31 +1,38 @@
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2009, 2020, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Implementing communication with MySQL servers.
 """
 
 from io import IOBase
 import os
+import platform
+import socket
 import time
 
 from .authentication import get_auth_plugin
@@ -35,7 +42,7 @@ from .constants import (
     flag_is_set, ShutdownType, NET_BUFFER_LENGTH
 )
 
-from . import errors
+from . import errors, version
 from .conversion import MySQLConverter
 from .cursor import (
     CursorBase, MySQLCursor, MySQLCursorRaw,
@@ -44,7 +51,7 @@ from .cursor import (
     MySQLCursorBufferedNamedTuple)
 from .network import MySQLUnixSocket, MySQLTCPSocket
 from .protocol import MySQLProtocol
-from .utils import int4store
+from .utils import int4store, linux_distribution
 from .abstracts import MySQLConnectionAbstract
 
 
@@ -59,7 +66,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._converter_class = MySQLConverter
 
         self._client_flags = ClientFlag.get_default()
-        self._charset_id = 33
+        self._charset_id = 45
         self._sql_mode = None
         self._time_zone = None
         self._autocommit = False
@@ -90,6 +97,8 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._auth_plugin = None
         self._pool_config_version = None
 
+        self._columns_desc = []
+
         if kwargs:
             try:
                 self.connect(**kwargs)
@@ -99,6 +108,39 @@ class MySQLConnection(MySQLConnectionAbstract):
                 self._socket = None
                 raise
 
+    def _add_default_conn_attrs(self):
+        """Add the default connection attributes."""
+        if os.name == "nt":
+            if "64" in platform.architecture()[0]:
+                platform_arch = "x86_64"
+            elif "32" in platform.architecture()[0]:
+                platform_arch = "i386"
+            else:
+                platform_arch = platform.architecture()
+            os_ver = "Windows-{}".format(platform.win32_ver()[1])
+        else:
+            platform_arch = platform.machine()
+            if platform.system() == "Darwin":
+                os_ver = "{}-{}".format("macOS", platform.mac_ver()[0])
+            else:
+                os_ver = "-".join(linux_distribution()[0:2])
+
+        license_chunks = version.LICENSE.split(" ")
+        if license_chunks[0] == "GPLv2":
+            client_license = "GPL-2.0"
+        else:
+            client_license = "Commercial"
+        default_conn_attrs = {
+            "_pid": str(os.getpid()),
+            "_platform": platform_arch,
+            "_source_host": socket.gethostname(),
+            "_client_name": "mysql-connector-python",
+            "_client_license": client_license,
+            "_client_version": ".".join([str(x) for x in version.VERSION[0:3]]),
+            "_os": os_ver}
+
+        self._conn_attrs.update((default_conn_attrs))
+
     def _do_handshake(self):
         """Get the handshake from the MySQL server"""
         packet = self._socket.recv()
@@ -109,8 +151,9 @@ class MySQLConnection(MySQLConnectionAbstract):
         try:
             handshake = self._protocol.parse_handshake(packet)
         except Exception as err:
-            raise errors.InterfaceError(
-                'Failed parsing handshake; {0}'.format(err))
+            # pylint: disable=E1101
+            raise errors.get_mysql_exception(msg=err.msg, errno=err.errno,
+                                             sqlstate=err.sqlstate)
 
         self._server_version = self._check_server_version(
             handshake['server_version_original'])
@@ -129,7 +172,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._handshake = handshake
 
     def _do_auth(self, username=None, password=None, database=None,
-                 client_flags=0, charset=33, ssl_options=None):
+                 client_flags=0, charset=45, ssl_options=None, conn_attrs=None):
         """Authenticate with the MySQL server
 
         Authentication happens in two parts. We first send a response to the
@@ -144,12 +187,18 @@ class MySQLConnection(MySQLConnectionAbstract):
             packet = self._protocol.make_auth_ssl(charset=charset,
                                                   client_flags=client_flags)
             self._socket.send(packet)
+            if ssl_options.get('tls_ciphersuites') is not None:
+                tls_ciphersuites = ":".join(ssl_options.get('tls_ciphersuites'))
+            else:
+                tls_ciphersuites = ""
             self._socket.switch_to_ssl(ssl_options.get('ca'),
                                        ssl_options.get('cert'),
                                        ssl_options.get('key'),
                                        ssl_options.get('verify_cert') or False,
-                                       ssl_options.get('cipher'),
-                                       ssl_options.get('version', None))
+                                       ssl_options.get('verify_identity') or
+                                       False,
+                                       tls_ciphersuites,
+                                       ssl_options.get('tls_versions'))
             self._ssl_active = True
 
         packet = self._protocol.make_auth(
@@ -157,7 +206,8 @@ class MySQLConnection(MySQLConnectionAbstract):
             username=username, password=password, database=database,
             charset=charset, client_flags=client_flags,
             ssl_enabled=self._ssl_active,
-            auth_plugin=self._auth_plugin)
+            auth_plugin=self._auth_plugin,
+            conn_attrs=conn_attrs)
         self._socket.send(packet)
         self._auth_switch_request(username, password)
 
@@ -234,17 +284,21 @@ class MySQLConnection(MySQLConnectionAbstract):
         """
         self._protocol = MySQLProtocol()
         self._socket = self._get_connection()
-        self._socket.open_connection()
-        self._do_handshake()
-
-        self._do_auth(self._user, self._password,
-                      self._database, self._client_flags, self._charset_id,
-                      self._ssl)
-
-        self.set_converter_class(self._converter_class)
-        if self._client_flags & ClientFlag.COMPRESS:
-            self._socket.recv = self._socket.recv_compressed
-            self._socket.send = self._socket.send_compressed
+        try:
+            self._socket.open_connection()
+            self._do_handshake()
+            self._do_auth(self._user, self._password,
+                          self._database, self._client_flags, self._charset_id,
+                          self._ssl, self._conn_attrs)
+            self.set_converter_class(self._converter_class)
+            if self._client_flags & ClientFlag.COMPRESS:
+                self._socket.recv = self._socket.recv_compressed
+                self._socket.send = self._socket.send_compressed
+            self._socket.set_connection_timeout(None)
+        except:
+            # close socket
+            self.close()
+            raise
 
     def shutdown(self):
         """Shut down connection to MySQL Server.
@@ -386,6 +440,8 @@ class MySQLConnection(MySQLConnectionAbstract):
         """Handle a LOAD DATA INFILE LOCAL request"""
         try:
             data_file = open(filename, 'rb')
+            return self._handle_ok(self._send_data(data_file,
+                                                   send_empty_packet=True))
         except IOError:
             # Send a empty packet to cancel the operation
             try:
@@ -395,9 +451,11 @@ class MySQLConnection(MySQLConnectionAbstract):
                     "MySQL Connection not available.")
             raise errors.InterfaceError(
                 "File '{0}' could not be read".format(filename))
-
-        return self._handle_ok(self._send_data(data_file,
-                                               send_empty_packet=True))
+        finally:
+            try:
+                data_file.close()
+            except (IOError, NameError):
+                pass
 
     def _handle_result(self, packet):
         """Handle a MySQL Result
@@ -433,16 +491,16 @@ class MySQLConnection(MySQLConnectionAbstract):
         if not column_count or not isinstance(column_count, int):
             raise errors.InterfaceError('Illegal result set.')
 
-        columns = [None,] * column_count
+        self._columns_desc = [None,] * column_count
         for i in range(0, column_count):
-            columns[i] = self._protocol.parse_column(
+            self._columns_desc[i] = self._protocol.parse_column(
                 self._socket.recv(), self.python_charset)
 
         eof = self._handle_eof(self._socket.recv())
         self.unread_result = True
-        return {'columns': columns, 'eof': eof}
+        return {'columns': self._columns_desc, 'eof': eof}
 
-    def get_row(self, binary=False, columns=None):
+    def get_row(self, binary=False, columns=None, raw=None):
         """Get the next rows returned by the MySQL server
 
         This method gets one row from the result set after sending, for
@@ -452,12 +510,14 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns a tuple.
         """
-        (rows, eof) = self.get_rows(count=1, binary=binary, columns=columns)
+        (rows, eof) = self.get_rows(count=1, binary=binary, columns=columns,
+                                    raw=raw)
         if rows:
             return (rows[0], eof)
         return (None, eof)
 
-    def get_rows(self, count=None, binary=False, columns=None):
+    def get_rows(self, count=None, binary=False, columns=None, raw=None,
+                 prep_stmt=None):
         """Get all rows returned by the MySQL server
 
         This method gets all rows returned by the MySQL server after sending,
@@ -466,6 +526,9 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns a tuple()
         """
+        if raw is None:
+            raw = self._raw
+
         if not self.unread_result:
             raise errors.InternalError("No result set available.")
 
@@ -484,13 +547,19 @@ class MySQLConnection(MySQLConnectionAbstract):
             self.unread_result = False
             raise err
 
-        if rows[-1] is not None:
-            row = rows[-1]  # OK or EOF
-            self._handle_server_status(row['status_flag'] if 'status_flag' in
-                                       row else row['server_status'])
+        rows, eof_p = rows
+
+        if not (binary or raw) and self._columns_desc is not None and rows \
+           and hasattr(self, 'converter'):
+            row_to_python = self.converter.row_to_python
+            rows = [row_to_python(row, self._columns_desc) for row in rows]
+
+        if eof_p is not None:
+            self._handle_server_status(eof_p['status_flag'] if 'status_flag' in
+                                       eof_p else eof_p['server_status'])
             self.unread_result = False
 
-        return rows
+        return rows, eof_p
 
     def consume_results(self):
         """Consume results
@@ -667,7 +736,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         return self._handle_ok(self._send_cmd(ServerCmd.PING))
 
     def cmd_change_user(self, username='', password='', database='',
-                        charset=33):
+                        charset=45):
         """Change the current logged in user
 
         This method allows to change the current logged in user information.
