@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
@@ -1182,8 +1182,6 @@ MySQL_connect(MySQL *self, PyObject *args, PyObject *kwds)
 
     tmp_uint= self->connection_timeout;
     mysql_options(&self->session, MYSQL_OPT_CONNECT_TIMEOUT, (char*)&tmp_uint);
-    mysql_options(&self->session, MYSQL_OPT_READ_TIMEOUT, (char*)&tmp_uint);
-    mysql_options(&self->session, MYSQL_OPT_WRITE_TIMEOUT, (char*)&tmp_uint);
 
     if (ssl_disabled  != NULL && (PyBool_Check(ssl_disabled) && ssl_disabled  == Py_False)) {
         ssl_enabled= 1;
@@ -2628,8 +2626,15 @@ MySQL_fetch_row(MySQL *self)
         }
         else if (field_type == MYSQL_TYPE_BLOB)
         {
-            value= mytopy_string(row[i], field_lengths[i], field_flags,
-                                  charset, self->use_unicode);
+            if ((field_flags & BLOB_FLAG) && (field_flags & BINARY_FLAG))
+            {
+                value= BytesFromStringAndSize(row[i], field_lengths[i]);
+            }
+            else
+            {
+                value= mytopy_string(row[i], field_lengths[i], field_flags,
+                                     charset, self->use_unicode);
+            }
             PyTuple_SET_ITEM(result_row, i, value);
         }
         else if (field_type == MYSQL_TYPE_GEOMETRY)
