@@ -1,4 +1,4 @@
-# Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -47,6 +47,11 @@ from imp import load_source
 from functools import wraps
 from pkgutil import walk_packages
 
+try:
+    import _mysql_connector
+except ImportError:
+    pass
+
 LOGGER_NAME = "myconnpy_tests"
 LOGGER = logging.getLogger(LOGGER_NAME)
 PY2 = sys.version_info[0] == 2
@@ -72,7 +77,7 @@ except ImportError:
         LOGGER.error("Could not initialize Python's unittest module")
         sys.exit(1)
 
-from lib.cpy_distutils import get_mysql_config_info
+from lib.cpy_distutils import mysql_c_api_info
 
 SSL_AVAILABLE = True
 try:
@@ -902,7 +907,7 @@ def check_c_extension(exc=None):
             mysql_config = os.path.join(MYSQL_CAPI, 'bin', 'mysql_config')
         else:
             mysql_config = MYSQL_CAPI
-        lib_dir = get_mysql_config_info(mysql_config)['lib_dir']
+        lib_dir = mysql_c_api_info(mysql_config)['link_directories']
     elif os.path.isdir(MYSQL_CAPI):
         lib_dir = os.path.join(MYSQL_CAPI, 'lib')
     else:
@@ -935,3 +940,33 @@ def check_c_extension(exc=None):
     else:
         LOGGER.error("C Extension not available: %s", error_msg)
         sys.exit(1)
+
+def check_tls_versions_support(tls_versions):
+    """Check whether we can connect with given TLS version
+
+    Attempts a connection to a server using a specific TLS version but does not verify
+    which TLS version was used on the connection.
+
+    :param: List of TLS versions to test.
+    :return: List of supported TLS versions.
+    :rtype: list
+    """
+    settings = get_mysql_config()
+    if "socket" in settings:
+        settings.pop("socket")
+    if "unix_socket" in settings:
+            settings.pop("unix_socket")
+    supported_tls = []
+    try:
+        from mysql.connector import MySQLConnection
+        for tls_v in tls_versions:
+            try:
+                settings["tls_versions"] = [tls_v]
+                cnx = MySQLConnection(**settings)
+                cnx.close()
+                supported_tls.append(tls_v)
+            except:
+                pass
+    except ImportError:
+        pass
+    return supported_tls
