@@ -485,7 +485,7 @@ class MySQLServer(MySQLServerBase):
 
         # Extra SQL statements to execute after SQL scripts
         extra_sql = [
-            "CREATE DATABASE myconnpy;"
+            "CREATE DATABASE myconnpy CHARACTER SET utf8mb4;"
         ]
 
         if self._version > (5, 7, 5) and self._version < (5, 7, 21):
@@ -523,7 +523,7 @@ class MySQLServer(MySQLServerBase):
                     "CREATE USER IF NOT EXISTS 'root'@'::1';",
                     "GRANT ALL ON *.* TO 'root'@'::1' WITH GRANT OPTION;",
                     "CREATE USER IF NOT EXISTS mysqlxsys@localhost IDENTIFIED "
-                    "WITH mysql_native_password AS 'password' ACCOUNT LOCK;",
+                    "WITH mysql_native_password BY 'password' ACCOUNT LOCK;",
                     "GRANT SELECT ON mysql.user TO mysqlxsys@localhost;",
                     "GRANT SUPER ON *.* TO mysqlxsys@localhost;"
                 ])
