@@ -46,6 +46,7 @@ from . import PY2
 
 try:
     from mysql.connector.connection_cext import HAVE_CMYSQL, CMySQLConnection
+    from mysql.connector import cursor_cext
 except ImportError:
     # Test without C Extension
     CMySQLConnection = None
@@ -701,7 +702,6 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
                 isinstance(cnx._handshake[key], type_),
                 "type check failed for '{0}', expected {1}, we got {2}".format(
                     key, type_, type(cnx._handshake[key])))
-        self.assertRaises(errors.OperationalError, cnx._do_handshake)
 
         class FakeSocket(object):
             def __init__(self, packet):
@@ -1229,6 +1229,18 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
         exp = conn_id + 1
         self.assertGreaterEqual(cnx.connection_id, exp)
 
+    @tests.foreach_cnx(connection_timeout=1)
+    def test_connect_timeout(self):
+        """Test connect_timeout.
+
+        The connect_timeout should be applied only for establishing the
+        connection and not for all blocking socket operations.
+        """
+        cur = self.cnx.cursor()
+        cur.execute("SELECT SLEEP(2)")
+        cur.fetchall()
+        cur.close()
+
     def test_ping(self):
         """Ping the MySQL server"""
         supported_arguments = {
@@ -1775,7 +1787,7 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
         data_file = os.path.join("tests", "data", "local_data.csv")
         cur = self.cnx.cursor()
         sql = "LOAD DATA LOCAL INFILE %s INTO TABLE local_data"
-        self.assertRaises(errors.ProgrammingError,
+        self.assertRaises((errors.DatabaseError, errors.ProgrammingError),
                           cur.execute, sql, (data_file,))
         cur.execute("DROP TABLE IF EXISTS local_data")
         cur.close()
@@ -1990,7 +2002,6 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
                 cur = cnx.cursor()
                 cur.execute("SHOW STATUS LIKE 'Ssl_version%'")
                 res = cur.fetchall()
-    
                 self.assertEqual(res[0][1], expected_ssl_version,
                                  err_msg.format(expected_ssl_version, res))
 
@@ -2184,6 +2195,29 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
         self.assertRaises(InterfaceError, connect, **config)
         del config["unix_socket"]
 
+    @unittest.skipIf(not HAVE_CMYSQL, "C Extension not available")
+    def test_context_manager_cext(self):
+        """Test connection and cursor context manager using the C extension."""
+        config = tests.get_mysql_config().copy()
+        config["use_pure"] = False
+        with connect(**config) as conn:
+            self.assertTrue(conn.is_connected())
+            with conn.cursor() as cur:
+                self.assertIsInstance(cur, cursor_cext.CMySQLCursor)
+            self.assertIsNone(cur._cnx)
+        self.assertFalse(conn.is_connected())
+
+    def test_context_manager_pure(self):
+        """Test connection and cursor context manager using pure Python."""
+        config = tests.get_mysql_config().copy()
+        config["use_pure"] = True
+        with connect(**config) as conn:
+            self.assertTrue(conn.is_connected())
+            with conn.cursor() as cur:
+                self.assertIsInstance(cur, cursor.MySQLCursor)
+            self.assertIsNone(cur._connection)
+        self.assertFalse(conn.is_connected())
+
 
 class WL13335(tests.MySQLConnectorTests):
     """WL#13335: Avoid set config values whit flag CAN_HANDLE_EXPIRED_PASSWORDS
